@@ -2,7 +2,7 @@ package Search::Typesense;
 
 use Moo;
 
-use 5.10.0;
+use v5.16.0;
 use Mojo::JSON qw(decode_json encode_json);
 use Mojo::UserAgent;
 use Mojo::URL;
@@ -19,6 +19,7 @@ Search::Typesense - Perl interface to Typesense search engine.
         use_https => $bool,
         host      => $host,
         port      => $port,
+        api_key   => $key,
     );
     
     my $results = $typesense->search(
@@ -55,44 +56,49 @@ use Search::Typesense::Types qw(
   HashRef
   InstanceOf
   NonEmptyStr
+  PositiveInt
   Str
   compile
 );
 
 =head1 VERSION
 
-Version 0.01
+Version 0.02
 
 =cut
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 
 has _ua => (
-    is      => 'ro',
+    is      => 'lazy',
     isa     => InstanceOf ['Mojo::UserAgent'],
-    lazy    => 1,
-    builder => 1,
+    builder => sub {
+        my $self = shift;
+        my $ua   = Mojo::UserAgent->new;
+        my $key  = $self->api_key;
+        $ua->on(
+            start => sub {
+                my ( $ua, $tx ) = @_;
+                $tx->req->headers->header(
+                    'Content-Type' => 'application/json' )
+                  ->header( 'X-TYPESENSE-API-KEY' => $key );
+            }
+        );
+        return $ua;
+    },
 );
 
-sub _build__ua {
-    my $self = shift;
-    my $ua   = Mojo::UserAgent->new;
-    $ua->on(
-        start => sub {
-            my ( $ua, $tx ) = @_;
-            $tx->req->headers->header( 'Content-Type' => 'application/json' );
-            $tx->req->headers->header(
-                'X-TYPESENSE-API-KEY' => $self->api_key );
-        }
-    );
-    return $ua;
-}
-
 has _url_base => (
-    is      => 'ro',
+    is      => 'lazy',
     isa     => InstanceOf ['Mojo::URL'],
-    lazy    => 1,
-    builder => 1,
+    builder => sub {
+        my $self = shift;
+        my $url  = Mojo::URL->new;
+        $url->scheme( $self->use_https ? 'https' : 'http' );
+        $url->host( $self->host );
+        $url->port( $self->port );
+        return $url;
+    },
 );
 
 has use_https => (
@@ -115,20 +121,11 @@ has host => (
 );
 
 has port => (
-    is       => 'ro',
-    isa      => NonEmptyStr,
-    required => 1,
+    is      => 'ro',
+    isa     => PositiveInt,
+    default => 8108,
 );
 
-sub _build__url_base {
-    my $self = shift;
-    my $url  = Mojo::URL->new;
-    $url->scheme( $self->use_https ? 'https' : 'http' );
-    $url->host( $self->host );
-    $url->port( $self->port );
-    return $url;
-}
-
 sub _url {
     my ( $self, $path ) = @_;
     return $self->_url_base->clone->path( '/' . join( '/' => @$path ) );
@@ -139,6 +136,33 @@ sub BUILD {
     $self->assert_is_running;
 }
 
+=head1 CONSTRUCTOR
+
+The constructor takes a list (or hashref) of key/value pairs.
+
+    my $typesense = Search::Typesense->new(
+        use_https => $bool,
+        host      => $host,
+        port      => $port,
+        api_key   => $key,
+    );
+
+=head2 C<api_key>
+
+The api key to which will be sent as the C<X-TYPESENSE-API-KEY> header.
+
+=head2 C<host>
+
+The hostname to connect to.
+
+=head2 C<port>
+
+Optional port number to connect to. Defaults to 8108 if not supplied.
+
+=head2 C<use_https>
+
+Optional boolean. Whether or not to connect to Typesense over https. Default true.
+
 =head1 METHODS
 
 =head2 C<assert_is_running>
@@ -223,7 +247,7 @@ sub create_collection {
     ($collection_definition) = $check->($collection_definition);
     my $fields = $collection_definition->{fields};
 
-    foreach my $field ( $fields->@* ) {
+    foreach my $field (@$fields) {
         if ( exists $field->{facet} ) {
             $field->{facet} =
               $field->{facet} ? Mojo::JSON->true : Mojo::JSON->false;
@@ -239,12 +263,7 @@ sub create_collection {
 sub _POST {
     my ( $self, %arg_for ) = @_;
     my $request = $arg_for{request};
-    my $url     = $self->_url( $arg_for{path} );
-
-    if ( my $query = $arg_for{query} ) {
-        my $parameters = Mojo::Parameters->new(%$query);
-        $url .= "?$parameters";
-    }
+    my $url     = $self->_url( $arg_for{path} )->query( $arg_for{query} || {} );
 
     my @args = ref $request
       ? ( json => $request )    # data structures
@@ -308,13 +327,27 @@ sub update_document {
     state $check = compile( NonEmptyStr, NonEmptyStr, HashRef );
     ( $collection, $document_id, $updates ) =
       $check->( $collection, $document_id, $updates );
-    return $self->_make_request(
-        method  => 'patch',
+    return $self->_PATCH(
         path    => [ 'collections', $collection, 'documents', $document_id ],
         request => $updates
     );
 }
 
+# Yet, we're getting some serious duplication here. Need to refactorit.
+sub _PATCH {
+    my ( $self, %arg_for ) = @_;
+    my $request = $arg_for{request};
+    my $url     = $self->_url( $arg_for{path} )->query( $arg_for{query} || {} );
+
+    my @args = ref $request
+      ? ( json => $request )    # data structures
+      : $request ? $request     # raw POST content
+      :            croak("PATCH $url not allowed without a request");
+
+    my $tx = $self->_ua->patch( $url, @args );
+    return $self->_check_for_failure( $tx, $arg_for{return_transaction} );
+}
+
 =head2 C<delete_document>
 
     my $document = $typesense->delete_document($collection_name, $document_id);
@@ -360,7 +393,7 @@ sub search {
         return_transaction => 1,
     ) or return;
     my $response = $tx->res->json;
-    foreach my $hit ( $response->{hits}->@* ) {
+    foreach my $hit ( @{ $response->{hits} } ) {
         if ( exists $hit->{document}{json} ) {
             $hit->{document}{json} = decode_json( $hit->{document}{json} );
         }
@@ -426,15 +459,15 @@ sub import_documents {
     return $response;
 }
 
-=head2 C<purge>
+=head2 C<delete_all_collections>
 
-    $typesense->purge;
+    $typesense->delete_all_collections;
 
 Deletes everything from Typsense. B<Use with caution>!
 
 =cut
 
-sub purge {
+sub delete_all_collections {
     my ($self) = @_;
     my $collections = $self->get_collections;
     foreach my $collection (@$collections) {
@@ -496,6 +529,7 @@ L<https://metacpan.org/release/Search-Typesense>
 
 =head1 ACKNOWLEDGEMENTS
 
+Thanks for Sebastian Reidel and Matt Trout for feedback.
 
 =head1 LICENSE AND COPYRIGHT
 
